Documentation is essential for new learners and experienced programmers alike. It helps make our community inclusive by extending a friendly hand to those that are less familiar with p5.js. It also helps us find the bugs and issues with the code itself, because we test and try things out as we document.

There are several ways to contribute to documentation:

## ✏️ Contribute to the reference  
Read through the [reference](http://p5js.org/reference/), and look for typos, broken examples, confusing documentation. If it's a straightforward fix, go ahead and work on it! If it's a more involved question that requires discussion, create an [issue](https://github.com/processing/p5.js/issues/new).
* Here are instructions for [getting setup for the first time with the p5.js repo](https://github.com/processing/p5.js/tree/master/developer_docs).
* The reference is built from the inline documentation in the source code (found in the src/ folder).
* Here is information on [how to update or add inline documentation and examples](https://github.com/processing/p5.js/blob/master/developer_docs/inline_documentation.md).
* If you find errors with the [spanish documentation](http://p5js.org/es), there are instructions to update this [here](https://github.com/processing/p5.js-website#internationalization-i18n-and-structure).

## ✨ Make examples  
While the examples in the reference are meant to be very simplistic snippets of code, it is also useful to have longer, more complex examples.
* Currently we are working on porting the examples from the [processing examples page](https://processing.org/examples/) to the [p5.js examples page](http://p5js.org/examples). If you would like to help with this, please see the instructions [here](https://github.com/processing/p5.js-website/wiki/Adding-examples).
* Alternatively, you can create your own set of examples and publish them independently anywhere you like. If you share them online tag [@p5xjs](https://twitter.com/p5xjs) or email [hello@p5js.org](mailto:hello@p5js.org) to let us know, and we will share far and wide! This [guide to embedding p5.js](https://github.com/processing/p5.js/wiki/Embedding-p5.js) may be useful for posting your examples online.
* If you find bugs in p5.js along the way, please log them in the [issues](https://github.com/processing/p5.js/issues).

## 👯 Make tutorials 
* If you are new to p5.js, this is a great place to start. Try making something yourself, then making a tutorial to teach others to do it too.
* Currently we are working on porting the tutorials from the [processing tutorials page](https://processing.org/tutorials) to the [p5.js learn page](http://p5js.org/learn). If you would like to help with this, please see the tutorial on making tutorials [here](https://p5js.org/learn/tutorial-guide.html).
* We also welcome tutorials on different subjects. You can publish these anywhere, in any format you like. If you share them online tag [@p5xjs](https://twitter.com/p5xjs) or email [hello@p5js.org](mailto:hello@p5js.org) to let us know, and we will share far and wide! We especially encourage tutorials created with a specific audience or use case in mind (ex: p5 for journalists, activists, poets, kids, seniors, dreamers, in a different language, etc). We like to consider, who does not already feel welcome or included in the p5 community, and can we use learning as an invitation? What is the language you use to communicate with your audience and make them feel known? Feel free to be experimental with your format. Check out [Sharon De La Cruz's talk on Code Slang](https://www.youtube.com/watch?v=CFT6w9NKfCs) for inspiration.
* If you find bugs in p5.js along the way, please log them in the [issues](https://github.com/processing/p5.js/issues).

## 👉 Getting started
* Read the [p5.js community statement](http://p5js.org/community/)
* Optional: Check out the [developer docs](https://github.com/processing/p5.js/tree/master/developer_docs) to get an overview of the repository and learn how to build the code (if relevant for you).
* All discussion happens on github issues, so there's no slack/gitter/etc channel you need to join.
* Add your name to the [contributors list](https://github.com/processing/p5.js#contributors) in the readme.md file! Instructions [here](https://github.com/processing/p5.js/issues/2309).
* And of course, if you're more of a bug fixer kind of person, feel free to jump into any of the [issues](https://github.com/processing/p5.js/issues)!

Welcome! We're so glad you're here!
❤️ the p5.js community
